import os

# Base directory for the project
base_dir = os.path.dirname(os.path.abspath(__file__))

first_gui_model_path = os.path.join(base_dir, "llm_assistant", "images", "library_directory.png")
plantuml_code_example_path = os.path.join(base_dir, "llm_assistant", "plantuml", "library.puml")
